/*

DESCRIPTION:
    Main source file for OAT module.

SYNTAX:  n/a

KNOWN BUGS:  n/a

COPYRIGHT:  (c) 2018, Brian Cornell, all rights reserved.

DEPENDENCIES:
    config.h - PIC configuration registers
    main.h - configuration constants for main.c

VERSION:  1.0, development release

MODIFICATION HISTORY:

    26 Jan 18, BWC, created file.
    

*/

//Config PIC configuration registers first
#include "config.h"

//System includes second
#include <pic.h>
#include <xc.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>

//User includes last
#include <binary.h>
#include "main.h"


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                              Data objects
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
const unsigned char ChrMap[] = {LCD_CHR_0,LCD_CHR_1,LCD_CHR_2,LCD_CHR_3,LCD_CHR_4,\
LCD_CHR_5,LCD_CHR_6,LCD_CHR_7,LCD_CHR_8,LCD_CHR_9,LCD_CHR_BLK};



/*Function main.
 *
 * Returns:  none
 *
 * Parameters:  none
 *
*/
void main(void)
{
    int result;
    signed char dsp;
    unsigned char ones, tens, hdrd, neg, count;

    //Configure system and pause for settle time
    Initialize();
    __delay_ms(S_TM_INT);

    SWDTEN = TRUE;                          //Enable watchdog timer

    //Infinite loop to get & display OAT.
    while (TRUE)
    {
        //Start / wake up from sleep...
        PIN_O_TMPWR = TRUE;                 //Power up temp sensor
        ADCON0bits.ON = TRUE;               //Enable ADC module
        S_EN_UNITS = TRUE;                  //Power up unit select pin
        __delay_ms(S_TM_SETTLE);            //Settle & acquisition time

        //Take multiple temp samples & average
        for (count = 0, result = 0; (count < S_QY_TMP_SAMPLES); ++count)
        {
            __delay_ms(5);                  //Acquisition time before next
            ADCON0bits.GO = TRUE;           //Start conversion
            while (ADCON0bits.GO);          //Wait for completion
            if (count > 0) result += ADRES; //Discard first conversion
        }
        ADCON0bits.ON = FALSE;              //Disable ADC module
        PIN_O_TMPWR = FALSE;                //Power down temp sensor

        result /= (S_QY_TMP_SAMPLES - 1);   //Calculate average
        //Calculate degrees C:  Ta=result/Tcoeff-T0c_offset
        dsp = (signed char) ((result / S_QY_TMP_COEFF) - S_QY_TMP_0COFFS);

        //Convert to F if selected
        if (PIN_I_UNITS) dsp = (((dsp * S_QY_F_CONVN) / S_QY_F_CONVD) + S_QY_F_OFFS);
        S_EN_UNITS = FALSE;                 //Power down unit select pin

        //Set / clear negative flag & convert dsp to + value if negative
        if (dsp < 0)
        {
            neg = LCD_CHR_NEG;
            dsp = ((dsp ^ 0xFF) + 1);       //Take absolute value
        }
        else neg = LCD_CHR_POS;

        //Separate dsp into individual values for 100, 10, 1 places
        if (!(hdrd = dsp / 100)) hdrd = LCD_MAP_BLK;
        else dsp -= 100;
        if ((!(tens = dsp / 10)) && (hdrd == LCD_MAP_BLK)) tens = LCD_MAP_BLK;
        ones = dsp % 10;

        //Wait until LCD module is ready for update & load display registers...
        while (!(LCDPSbits.WA));
        LCDDATA0 = ChrMap[ones];
        LCDDATA1 = ChrMap[tens];
        //Merge negative sign with 100's place & shift to accommodate register
        LCDDATA2 = (neg | (ChrMap[hdrd] << 1));

        SLEEP();                            //Sleep until watchdog wakes up

    }
} //End function main


/*Function Initialize.  Performs baseline MPU initialization at boot.
 *
 * Returns:  none
 *
 * Parameters:  none
 *
*/
void Initialize(void)
{
    di();                               //No interrupts

    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Set port configuration.  To prevent any aberrant behaviors before 
     * initialization is complete set output latches.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    //Port A
    PORTA = VL_PORTA;
    TRISA = IO_PORTA;
    ANSELA = AD_PORTA;
    WPUA = WP_PORTA;
    ODCONA = OD_PORTA;
    SLRCONA = SR_PORTA;
    INLVLA = IL_PORTA;
    //Port B
    PORTB = VL_PORTB;
    TRISB = IO_PORTB;
    ANSELB = AD_PORTB;
    WPUB = WP_PORTB;
    ODCONB = OD_PORTB;
    SLRCONB = SR_PORTB;
    INLVLB = IL_PORTB;
    //Port C
    PORTC = VL_PORTC;
    TRISC = IO_PORTC;
    ANSELC = AD_PORTC;
    WPUC = WP_PORTC;
    ODCONC = OD_PORTC;
    SLRCONC = SR_PORTC;
    INLVLC = IL_PORTC;
    //Port E
    PORTE = VL_PORTE;
    WPUE = WP_PORTE;
    INLVLE = IL_PORTE;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Power-down unneeded peripherals to save energy
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    PMD0 = CFG_PMD_0;
    PMD1 = CFG_PMD_1;
    PMD2 = CFG_PMD_2;
    PMD3 = CFG_PMD_3;
    PMD4 = CFG_PMD_4;
    PMD5 = CFG_PMD_5;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Configure LCD module
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    LCDCON = CFG_LCD_CON;
    LCDPS = CFG_LCD_PS;
    LCDSE0 = CFG_LCD_SEG0;
    LCDSE1 = CFG_LCD_SEG1;
    LCDSE2 = CFG_LCD_SEG2;
    LCDVCON2 = CFG_LCD_VC2;
    LCDDATA0 = CFG_LCD_DAT0;
    LCDDATA1 = CFG_LCD_DAT1;
    LCDDATA2 = CFG_LCD_DAT2;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Configure FVR
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    FVRCON = CFG_FVR_CON;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Configure ADC
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    ADCON0 = CFG_ADC_CON0;
    ADCON1 = CFG_ADC_CON1;
    ADCON2 = CFG_ADC_CON2;
    ADCON3 = CFG_ADC_CON3;
    ADCLK = CFG_ADC_CLK;
    ADREF = CFG_ADC_REF;
    ADPCH = CFG_ADC_PCH;
    ADPRE = CFG_ADC_PRE;
    ADACQ = CFG_ADC_ACQ;
    ADCAP = CFG_ADC_CAP;
    ADRPT = CFG_ADC_RPT;
    ADACT = CFG_ADC_ACT;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Configure watchdog timer
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    WDTCON0 = CFG_WCHD_CON0;
    WDTCON1 = CFG_WCHD_CON1;


    PEIE = ENABLE;                      //Enable peripheral interrupts
    LCDEN = TRUE;                       //Enable LCD module

    return;
} //End function Initialize



//End of file